///////////////////////////////////////////////////////////////////////
// Dynamically loading sub-levels.
// by Dirk Mittler.		August 11, 2005
//		Updated:				January 24, 2007

// To conserve main memory, this script will load Map Levels
// in response to players being within range of 'createboxes' .
// Optionally, Fog or Clip range is used to calculate that range.
// An arbitrary distance can otherwise be used.
// For each sublevel MAP ENTITY, copy the outer boundary block
// into a separate MAP ENTITY called a Createbox, and write a stub
// for it as shown here after main code.

// The origin of sublevel and createbox must match exactly !!!

define _CBOX_RANGE,SKILL1;		// You may optionally preset.
define _CBOX_IDLE,SKILL2;
define _CBOX_STRINGH,SKILL3;

define _CBOX_ONCEH,SKILL4;
				// Action called only once for sublevel.
define _CBOX_AGAINH,SKILL5;
				// Action called every time sublevel recreated.
define _CBOX_DESTRH,SKILL6;
				// Action which removes panels scripted in _AGAINH.

define _CBOX_ACTIVE,FLAG1;
define _CBOX_RUNNING,FLAG2;	// DO NOT SET in WED!

define CBOX_PLAYER_RANGE, 1500;
// If Createboxes don't preclude rendering distant details.
// (Great for outdoor games where all Terrain exists permanently.
// Like the Createboxes themselves, put Terrain directly into
// Game Level.

// Comment out def CBOX_PLAYER_RANGE,
// if using Createboxes means that visibility is limited to
// loaded Sublevels, or that all Sublevels will load within
// maximum visibility range. (Great for cave- or indoors games.
// Any distant Terrain etc. must be virtual beyond real 3D, as
// Sky scene cylinders...)

var sublevel_timeout = 160;	// Sub-level released after 10 secs.


string createbox_str = "                                    ";
action* cb_tempact;
entity* temp_createbox;

function sublevel_function() {
	my.parent = you;

	cb_tempact = ptr_for_handle(your._CBOX_AGAINH);
	cb_tempact();

	while(me != null) {
		temp_createbox = my.parent;
		if (temp_createbox._CBOX_ACTIVE == OFF) {
			cb_tempact = ptr_for_handle(temp_createbox._CBOX_DESTRH);
			cb_tempact();
			ent_remove(me);
		}
		wait (1);
	}
}

function createbox() {
	if (my._CBOX_RUNNING == ON) {return; }
	my._CBOX_RUNNING = ON;
	my.invisible = ON;
	my.passable = ON;
			// Just to make sure.

	temp.x = max(my.min_x * my.min_x, my.max_x * my.max_x);
	temp.y = max(my.min_y * my.min_y, my.max_y * my.max_y);
	temp.z = max(my.min_z * my.min_z, my.max_z * my.max_z);
	if (my._CBOX_RANGE == 0) {
		my._CBOX_RANGE = sqrt(temp.x + temp.y + temp.z) + 400;
												// Allow for movement, lag.
	}

	my._CBOX_IDLE = sublevel_timeout + 1;
	my._CBOX_ACTIVE = OFF;

	sleep (0.5);		// Allow main level to load.
	wait(random(4));	// Randomize distr of scans.

ifndef CBOX_PLAYER_RANGE;
	if (camera.clip_far > 15500) {
		camera.clip_far = 1500;
							// Sorry. Limited visibility range
							// is a must for this script.
	}
endif;

	while (1) {
		temp.PAN = 360;
		temp.TILT = 180;

ifndef CBOX_PLAYER_RANGE;
		if (camera.fog_end == 0) {
			temp.Z = my._CBOX_RANGE + camera.clip_far;
		} else {
			temp.Z = my._CBOX_RANGE + camera.fog_end;
		}

ifelse;
		temp.z = my._CBOX_RANGE + CBOX_PLAYER_RANGE;
endif;

		indicator = 0;

ifdef ACKNEX_VERSION640;
		c_scan(my.POS, my.ANGLE, temp, SCAN_ENTS);
ifelse;
		scan_entity(my.POS, temp);
endif;

		wait (4);	// Reduce load on CPU.

		my._CBOX_IDLE += (TIME * 4);
		if (my._CBOX_IDLE > sublevel_timeout + 1) {
			my._CBOX_IDLE = sublevel_timeout + 1;
		}

		if (my._CBOX_ACTIVE == OFF && my._CBOX_IDLE < sublevel_timeout) {
			my._CBOX_ACTIVE = ON;
			str_cpy(createbox_str, ptr_for_handle(my._CBOX_STRINGH));
			you = ent_create(createbox_str, my.POS, sublevel_function);
										// !! Assumes same origin !!

			if (my._CBOX_ONCEH != 0) {
				cb_tempact = ptr_for_handle(my._CBOX_ONCEH);
				my._CBOX_ONCEH = 0;
				temp_createbox = you;
				cb_tempact();			// Create sublevel characters, etc.
			}
		}

		if (my._CBOX_ACTIVE == ON && my._CBOX_IDLE > sublevel_timeout) {
																// Avoid == .
			my._CBOX_ACTIVE = OFF;
		}
	}
}

action createbox_event {
	if (EVENT_TYPE == EVENT_DETECT && your._TYPE == _TYPE_PLAYER) {
		my._CBOX_IDLE = 0;
	}
}

/* Sample usage:

// To complete an appearance of visibility beyond the real 3D,
// Sky cylinders may be used. 1 Sky cylinder may be shared by
// numerous sublevels, but would still be intended as having
// a real distance for only certain eligible sublevels.

var sky20_usecount = 0;
var sky21_usecount = 0;

var sky20_dist = 5000;
var sky21_dist = 15000;

var sky20_compass = 215;	// Facing SW
var sky21_compass = 180;	// Facing due South
									// (Image Center)

var sky20_xorg = 5000;
var sky20_yorg = 5000;		// Shot from a NE point

var sky21_xorg = 0;
var sky21_yorg = 8000;		// Shot from a N point

function use_sky20() {
	sky20_usecount += 1;
	sky20.visible = ON;
}

function disuse_sky20() {
	sky20_usecount -= 1;
	if (sky20_usecount < 0.5) {
		sky20_usecount = 0;
		sky20.visible = OFF;
	}
}

function use_sky21() {
	sky21_usecount += 1;
	sky21.visible = ON;
}

function disuse_sky21() {
	sky21_usecount -= 1;
	if (sky21_usecount < 0.5) {
		sky21_usecount = 0;
		sky21.visible = OFF;
	}
}

//	...

///////////////////////////////////////////////////////////////////////
//
entity* canyon_cbent;

string canyon_wmb = <canyon.wmb>;	// Max 36 chars w extension.

action canyon_trolls { sleep (0.5); }	// ent_create() them here.
			// You may copy temp_createbox FIRST as sublevel-pointer.
			// But, sublevel will vanish.
			// Use my._CBOX_ACTIVE to suspend actions...
			// my.parent = you = createbox inside ent_create().

action canyon_details {	// Run by (parent) sublevel
	use_sky20();			// each time recreated.
	use_sky21();
}
			// You may prepose normal sublevel Map Entity action.
			
action canyon_destroy {
			// Destroy scripted panels... from canyon_details().
	disuse_sky20();
	disuse_sky21();
}

action cb_canyon {			// Assign in WED.
	canyon_cbent = me;
	my.ENABLE_DETECT = ON;
	my.EVENT = createbox_event;
	
	my._CBOX_STRINGH = handle(canyon_wmb);
	
	if (my._CBOX_RUNNING == OFF || my._CBOX_ONCEH != 0) {
		my._CBOX_ONCEH = handle(canyon_trolls);
	}

	my._CBOX_AGAINH = handle(canyon_details);
	my._CBOX_DESTRH = handle(canyon_destroy);
								// All 4 OBJECTS must exist.
	createbox();
}

function canyon_restore () {
	me = canyon_cbent;
	cb_canyon();
}

function cloak_canyon() {
	while(1) {
		wait(1);					// Allow ptrs to be set.

		if (canyon_cbent._CBOX_ACTIVE == OFF) {
			my.skill4 = 64;	// Allow 4 seconds to load.
			my.invisible = OFF;
		}
		my.skill4 -= TIME;
		if (my.skill4 < 0) {
			my.skill4 = 0;
			my.invisible = ON;
		}
	}
}

action sprite_bef_canyon {
	// Can be compiled into adjacent sublevel.
	my.oriented = ON;
	my.passable = ON;
	// Need to be set first.
	sleep(0.5);
	// Allow some level-loading.

	cloak_canyon();
}

//	...

///////////////////////////////////////////////////////////////////////
// This function must be called IMMEDIATELY AFTER load() when
// loading previously saved game, before any wait() instructions:

function restore_sublevels () {
	canyon_restore();		// Include function for each sublevel.
}

///////////////////////////////////////////////////////////////////////
// Must be called by Player Entity once:

function tilt_skies() {
	while (1) {
		if (sky20_usecount) {
			sky20.tilt = -atan(my.z / sky20_dist + 0.577);
																	// tan(+30 deg)
			temp = sky20_compass;
			sky20.pan = temp - 90 - (sky20.scale_x * 180) -
				atan( ((my.x - sky20_xorg) * cos(temp) -
							(my.y - sky20_yorg) * sin(temp)) /
						sky20_dist);
		}

		if (sky21_usecount) {
			sky21.tilt = -atan(my.z / sky21_dist + 0.577);
																	// tan(+30 deg)
			temp = sky21_compass;
			sky21.pan = temp - 90 - (sky21.scale_x * 180) -
				atan( ((my.x - sky21_xorg) * cos(temp) -
							(my.y - sky21_yorg) * sin(temp)) /
						sky21_dist);
		}
// Assuming each panorama was meant to show from 30 deg below horizon.
//		...
	
	wait(1);
	}
}

*/
